﻿/******************************************************************************
 * Copyright 2019 The siasun Transport Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 1.0 (the "License");
 *****************************************************************************/

#include "canbus.h"
#include"candatatype.h"
#include"messagecoder.h"
#include"../common/message.h"
#include"../common/canmsgobdmsgid.h"
#include "std_msgs/String.h"
#include <canbus/canMsg.h>


namespace  robot{
namespace canbusSpace {



std::string Canbus::Name() const { return "canbus"; }

int Canbus::Init() {
  LOG(INFO)<< Name()<<" Init start...";
  node_handle_.reset(new ros::NodeHandle());

  //load libpcan.so using dlopen function,return handle for further use
  libm_handle = dlopen("libpcan.so", RTLD_LAZY );
  if (!libm_handle){
      LOG(ERROR)<< Name()<<" Init Open Error:" << dlerror();//if file can't be loaded,return null,get reason using dlerror function
      return ROBOT_FAILTURE;
  }
  char *errorInfo;//error information pointer
  //one-to-one mapping using dlsym function,if return null,mapping would be failed
  funCAN_Init         =(funCAN_Init_TYPE)         dlsym(libm_handle,"CAN_Init");
  funLINUX_CAN_Open   =(funLINUX_CAN_Open_TYPE)   dlsym(libm_handle,"LINUX_CAN_Open");
  funCAN_Close        =(funCAN_Close_TYPE)        dlsym(libm_handle,"CAN_Close");
  funCAN_VersionInfo  =(funCAN_VersionInfo_TYPE)  dlsym(libm_handle,"CAN_VersionInfo");
  funLINUX_CAN_Read   =(funLINUX_CAN_Read_TYPE)   dlsym(libm_handle,"LINUX_CAN_Read");
  funCAN_Status       =(funCAN_Status_TYPE)       dlsym(libm_handle,"CAN_Status");
  funnGetLastError    =(funnGetLastError_TYPE)    dlsym(libm_handle,"nGetLastError");
  funCAN_Write        =(funCAN_Write_TYPE)       dlsym(libm_handle, "CAN_Write");

  errorInfo = dlerror();//get error using dlerror function,and clear the error list in memory
  if (errorInfo != NULL){
      LOG(ERROR)<<Name()<<" Init Dlsym Error:"<<errorInfo;
      return ROBOT_FAILTURE;
  }

  //initialize the VehHBrkStat as Hold
  status1.uiVehHBrkStat = 1;


  InitObd();

  LOG(INFO)<<Name()<<" init end.";
  return ROBOT_SUCCESS;
}



//打开Can总线
int Canbus::Start() {
  char chVersion[VERSIONSTRING_LEN];            //store information of can version
  const char  *szDevNode = DEFAULT_NODE;  //define const pointer point to device name
  pcan_handle = funLINUX_CAN_Open(szDevNode, O_RDWR );//use mapping function

  //judge whether the call is success.if pcan_handle=null,the call would be failed
  if(pcan_handle){
      LOG(INFO)<<Name()<<": "<<szDevNode<<" opened";
      errno = funCAN_VersionInfo(pcan_handle, chVersion);
      if (!errno)
          LOG(INFO)<<Name()<<": driver version ="<<chVersion;
      else {
          LOG(ERROR)<<Name()<<": CAN_VersionInfo() error";
      }
      if (unBaudrate) {
              errno = funCAN_Init(pcan_handle, unBaudrate, nExtended);
              if (errno) {
                  LOG(ERROR)<<Name()<<": CAN_Init()";
              }
              else
                  LOG(INFO)<<" Device Info: " << szDevNode <<"; CAN_BAUD_1M; CAN_INIT_TYPE_ST";
          }
  }
  else
  {
      LOG(INFO)<<Name()<<": can't open ."<<szDevNode;
  }

  ros::Rate loop_rate(1/rate);
  // 5. set timer to triger publish info periodly
  timer_ = CreateTimer(ros::Duration(duration), &Canbus::OnTimer, this);

  // this location is necessary

  canRCUData_pub = node_handle_->advertise<canbus::canMsg>(robot::common::CanDataAnalysis, 2000);
  canObdData_pub = node_handle_->advertise<canbus::obdMsg>(robot::common::ObdData, 2000);

  ros::ServiceServer ActuatorEnableService = node_handle_->advertiseService(robot::common::ActuatorEnableService,
                                                                &Canbus::ActuatorEnableRequest, this);
  ros::ServiceServer LearnModeService = node_handle_->advertiseService(robot::common::LearnModeService,
                                                            &Canbus::ExcuteLearnModeRequest, this);
  ros::ServiceServer ShiftActCtrService = node_handle_->advertiseService(robot::common::ShiftActCtrService,
                                                              &Canbus::SftActCtrRequest, this);
  ros::ServiceServer SteerActCtrService = node_handle_->advertiseService(robot::common::SteerActCtrService,
                                                              &Canbus::StrActCtrRequest, this);
  ros::ServiceServer PedalActCtrService = node_handle_->advertiseService(robot::common::PedalActCtrService,
                                                              &Canbus::PedalActCtrRequest, this);
  //data receive
  while(ros::ok())
  {
//       canbus::canMsg rosMsg ;
//       rosMsg.id = 10;
//       rosMsg.data = "test";
//       canRCUData_pub.publish(rosMsg);
//       LOG(INFO)<<Name()<<" Publish CanData :" << "ID:" << rosMsg.id << ",Data:"<<rosMsg.data;

      read_loop();
      canObdData_pub.publish(obd);
      ros::spinOnce();
      loop_rate.sleep();
   }
   return ROBOT_SUCCESS;
}




void Canbus::OnTimer(const ros::TimerEvent &) {
  LOG(INFO)<< Name()<< " timer start ..";
  if(heatbeat > 15)
  {
      heatbeat = 0;
  }
  icuMonitor.uiICUAlive = heatbeat++;
  heatbeat ++;
  TPCANMsg canStatus1; //VehicleStaus1 Can Message
  TPCANMsg canIcuMonitor; //ICUMonitor Can Message
  TPCANMsg canStatus2;  //VehicleStatus2 Can Message
  TPCANMsg canStatus3;  //VehicleStatus3 Can Message
  try
  {
      MessageCoder::CanMsgPackage((void*)&icuMonitor, robot::common::ICUMonitor, &canIcuMonitor);
      MessageCoder::CanMsgPackage((void*)&status1, robot::common::VehicleStatus1, &canStatus1);
      MessageCoder::CanMsgPackage((void*)&status2, robot::common::VehicleStatus2, &canStatus2);
      MessageCoder::CanMsgPackage((void*)&status3, robot::common::VehicleStatus3, &canStatus3);
      write_data(&canStatus1);
      write_data(&canStatus2);
      write_data(&canStatus3);
      write_data(&canIcuMonitor);

      T_STR_ACT_CTR_REQ strAct = {0};
      strAct.iStrAngCtrReq =  2324;
      strAct.iStrSpdCtrReq= 10000;
      strAct.iStrTrqCtrReq = 5000;
      strAct.uiStrCtrMdReq = 1;
      TPCANMsg pStr;
      MessageCoder::CanMsgPackage((void*)&strAct,robot::common::SteerActCtrReq,&pStr);
      write_data(&pStr);
  }
  catch (ros::Exception ex)
  {
       LOG(ERROR)<<Name()<< " OnTimer error. "<< ex.what();
  }
}

void Canbus::Stop() {

  //Must close h handle firstly,then close file using dlclose
  if (pcan_handle) {
      funCAN_Close(pcan_handle);
  }
  LOG(INFO)<<Name()<<": closed. ";
  ros::shutdown();
  node_handle_.release();
  LOG(INFO)<<Name()<<": node shutdown. ";
}


void Canbus::InitObd()
{
     obd.AccPsng = 0;
     obd.BrkPsng = 0;
     obd.SftPsng = 0;
     obd.StrAng = 0;
     obd.VehHBrkStat = 0;
     obd.VehSpd = 0;
     obd.VehSwStat = 0;
}

//read from CAN forever - until manual break
int Canbus::read_loop()
{

    TPCANRdMsg msg;
    __u32 status;

    if (funLINUX_CAN_Read(pcan_handle, &msg)) {
        LOG(ERROR)<<Name()<<" receive: LINUX_CAN_Read() error.";
        return errno;
    }
    read_data(&(msg.Msg));

     // deal receive data
    // check if a CAN status is pending
    if (msg.Msg.MSGTYPE & MSGTYPE_STATUS) {
        status = funCAN_Status(pcan_handle);
        if ((int)status < 0) {
            errno = funnGetLastError();
            LOG(ERROR)<<Name()<<" receive: CAN_Status() error.";
            return errno;
        }
    }

    return ROBOT_SUCCESS;
}

int Canbus::read_data(TPCANMsg *pMsgBuff)
{
//    char strId[4] = {0};
//    sprintf(strId,"%3xh",pMsgBuff->ID);
//    char strData[16]={0};
//    MessageCoder::ConvertUnCharToStr(strData,pMsgBuff->DATA,8);

//    LOG(INFO)<<Name()<<" Receive CanData : ID: "<<std::string(strId)<<",Data: "<<std::string(strData);
    if(pMsgBuff->ID == robot::common::ObdAccPsng)
    {
       T_OBD_ACC_PSNG tOBDAccPsng = *((T_OBD_ACC_PSNG*)MessageCoder::CanMsgAnalyse(robot::common::ObdAccPsng,pMsgBuff->DATA));
       status1.uiAccPsng = tOBDAccPsng.obdAccPsng;
       obd.AccPsng = status1.uiAccPsng * 0.1;
    }
    else if(pMsgBuff->ID == robot::common::ObdBrkPsng)
    {
       T_OBD_BRK_PSNG tOBDBrkPsng = *((T_OBD_BRK_PSNG*)MessageCoder::CanMsgAnalyse(robot::common::ObdBrkPsng,pMsgBuff->DATA));
       status1.uiBrkPsng = tOBDBrkPsng.obdBrkPsng;
       obd.BrkPsng =  status1.uiBrkPsng * 0.1;
    }
    else if(pMsgBuff->ID == robot::common::ObdSftPsng)
    {
       T_OBD_SFT_PSNG tOBDSftPsng = *((T_OBD_SFT_PSNG*)MessageCoder::CanMsgAnalyse(robot::common::ObdSftPsng,pMsgBuff->DATA));
       status1.uiSftPsng = tOBDSftPsng.obdSftPsng;
       obd.SftPsng = status1.uiSftPsng;
    }
    else if(pMsgBuff->ID == robot::common::ObdVehHBrkStat)
    {
        T_OBD_VEH_BRK_STAT tOBDVehBrkStat = *((T_OBD_VEH_BRK_STAT*)MessageCoder::CanMsgAnalyse(robot::common::ObdVehHBrkStat,pMsgBuff->DATA));
       obd.VehHBrkStat = status1.uiVehHBrkStat = tOBDVehBrkStat.obdVehHBrkStat;
    }
    else if(pMsgBuff->ID == robot::common::ObdVehSwStat)
    {
        T_OBD_VEH_SW_STAT tOBDVehSwStat = *((T_OBD_VEH_SW_STAT*)MessageCoder::CanMsgAnalyse(robot::common::ObdVehSwStat,pMsgBuff->DATA));
        obd.VehSwStat = status1.uiVehSwStat = tOBDVehSwStat.obdVehSwStat;
    }
    else if(pMsgBuff->ID == robot::common::ObdStrAng)
    {
       T_OBD_STR_ANG tOBDStrAng = *((T_OBD_STR_ANG*)MessageCoder::CanMsgAnalyse(robot::common::ObdStrAng,pMsgBuff->DATA));
       status2.iStrAng = tOBDStrAng.obdStrAng;
       obd.StrAng = status2.iStrAng * 0.01;
    }
    else if(pMsgBuff->ID == robot::common::ObdEngSpd)
    {
        //T_OBD_ENG_SPD tOBDEngSpd = *((T_OBD_ENG_SPD*)MessageCoder::CanMsgAnalyse(robot::common::ObdEngSpd,pMsgBuff->DATA));
        //status2.uiEngSpd = tOBDEngSpd.obdEngSpd;
    }
    else if((pMsgBuff->ID >= robot::common::PowerOnReq && pMsgBuff->ID <= robot::common::BrakeForceCtrStp) ||
            (pMsgBuff->ID >= robot::common::SysMonitorBack && pMsgBuff->ID <= robot::common::SteerPosTimeReq) ||
            (pMsgBuff->ID >= robot::common::RobotAcuatorPos && pMsgBuff->ID <= robot::common::ComputerStaus) ||
            (pMsgBuff->ID >= robot::common::SteerTranLineCtr && pMsgBuff->ID <= robot::common::VehicleStart) ||
            pMsgBuff->ID ==robot::common::EstopInfo || pMsgBuff->ID ==robot::common::EstopErr)
    {
        publish_data(pMsgBuff);

    }
    return ROBOT_SUCCESS;
}

void Canbus::publish_data(TPCANMsg *pMsgBuff)
{
    canbus::canMsg rosMsg;
    rosMsg.id = pMsgBuff->ID;
    std::stringstream ss;
    ss << pMsgBuff->DATA;
    rosMsg.data = ss.str();
    canRCUData_pub.publish(rosMsg);

    char strId[4] = {0};
    sprintf(strId,"%3xh",pMsgBuff->ID);
    char strData[16]={0};
    MessageCoder::ConvertUnCharToStr(strData,pMsgBuff->DATA,8);
    LOG(INFO)<<Name()<<" Publish CanData :" << "ID:" << strId << ",Data:"<<strData;
    LOG(INFO)<<Name()<<" Msg CanData :" << "ID:" << rosMsg.id << ",Data:"<<rosMsg.data;
}

int Canbus::write_data(TPCANMsg *pMsgBuff)
{
  char strId[4] = {0};
  sprintf(strId,"%3xh",pMsgBuff->ID);
  char strData[16]={0};
  MessageCoder::ConvertUnCharToStr(strData,pMsgBuff->DATA,8);
  LOG(INFO)<<Name()<<" send CanData : ID: "<<std::string(strId)<<",Data:"<<std::string(strData);
  if (funCAN_Write(pcan_handle, pMsgBuff)){
     LOG(ERROR)<<Name()<< " write: LINUX_CAN_write_timeout error. ";
     return errno;
  }
   return ROBOT_SUCCESS;
}


bool Canbus::ActuatorEnableRequest(canbus::ActuatorEnableRequest::Request &req,
                           canbus::ActuatorEnableRequest::Response &res)
{
    LOG(INFO)<< "controlservice start ActuatorEnableRequest";
    // actuator enable request
    TPCANMsg canActuaEnableReqMsg;
    try{
        T_ACTUATOR_ENABLE_REQUEST request;
        request.uiAccClchCtrReq = req.uiAccClchCtrReq;
        request.uiAccMotEnReq = req.uiAccMotEnReq;
        request.uiBrkMotEnReq = req.uiBrkMotEnReq;
        request.uiClchMotEnReq = req.uiClchMotEnReq;
        request.uiCtrMdReq= req.uiCtrMdReq;
        request.uiEstpEnReq = req.uiEstpEnReq;
        request.uiPauseEnReq = req.uiPauseEnReq;
        request.uiPmpEnReq = req.uiPmpEnReq;
        request.uiStrMotEnReq = req.uiStrMotEnReq;
        request.uiXSftMotEnReq = req.uiXSftMotEnReq;
        request.uiYSftMotEnReq = req.uiYSftMotEnReq;

        MessageCoder::CanMsgPackage((void*)&request, robot::common::ActuatorEnableReq, &canActuaEnableReqMsg);
        write_data(&canActuaEnableReqMsg);
        res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice ActuatorEnableRequest failed "<< ex.what();
        return false;
    }
    return true;
}


bool Canbus::ExcuteLearnModeRequest(canbus::LearnModeRequest::Request & req,
                           canbus::LearnModeRequest::Response &res)
{
    LOG(INFO)<< "controlservice start ExcuteLearnModeRequest";
    // learn mode request
    TPCANMsg canLearnModeReqMsg;
    try{
        T_LEARN_MODE_REQUEST request;
        request.uiClrAccTbl = req.uiClrAccTbl;
        request.uiClrBrkTbl = req.uiClrBrkTbl;
        request.uiClrClchTbl = req.uiClrClchTbl;
        request.uiClrSftTbl = req.uiClrSftTbl;
        request.uiClrStrTbl= req.uiClrStrTbl;
        request.uiPsngLrnReq = req.uiPsngLrnReq;
        request.uiSftAutLrnFnsConf = req.uiSftAutLrnFnsConf;
        request.uiSftPsngLrnReq = req.uiSftPsngLrnReq;
        request.uiStpAutLrn = req.uiStpAutLrn;
        request.uiSysLrnReq = req.uiSysLrnReq;

        MessageCoder::CanMsgPackage((void*)&request, robot::common::LearnModeReq, &canLearnModeReqMsg);
        write_data(&canLearnModeReqMsg);
        res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice ExcuteLearnModeRequest failed "<< ex.what();
        return false;
    }
    return true;
}


bool Canbus::PedalActCtrRequest(canbus::PedalActCtrReq::Request &req,
                        canbus::PedalActCtrReq::Response &res)
{
    LOG(INFO)<< "controlservice start PedalActCtrRequest";
    // PedalActCtr request
    TPCANMsg canPedalActCtrReqMsg;
    try{
        T_PEDAL_ACT_CTR_REQ request;
        request.iAccSpdCtrReq = req.iAccSpdCtrReq;
        request.iBrkSpdCtrReq = req.iBrkSpdCtrReq;
        request.uiAccPsngCtrReq = req.uiAccPsngCtrReq;
        request.uiBrkCtrMdReq = req.uiBrkCtrMdReq;
        request.uiBrkFocCtrReq = req.uiBrkFocCtrReq;
        request.uiBrkFocCtrReq = req.uiBrkFocCtrReq;
        request.uiBrkPsngCtrReq = req.uiBrkPsngCtrReq;

        MessageCoder::CanMsgPackage((void*)&request, robot::common::PedalActCtrReq, &canPedalActCtrReqMsg);
        write_data(&canPedalActCtrReqMsg);
        res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice PedalActCtrRequest failed "<< ex.what();
        return false;
    }
    return true;
}

bool Canbus::SftActCtrRequest(canbus::SftActCtrReq::Request &req,
                      canbus::SftActCtrReq::Response &res)
{
    LOG(INFO)<< "controlservice start SftActCtrRequest";
    // SftActCtr request
    TPCANMsg canSftActCtrReqMsg;
    try{
        T_SFT_ACT_CTR_REQ request;
        request.iClchSpdCtrReq = req.iClchSpdCtrReq;
        request.uiClchPsngCtrReq = req.uiClchPsngCtrReq;
        request.uiSftMdReq = req.uiSftMdReq;
        request.uiSftPsngCtrReq = req.uiSftPsngCtrReq;

        MessageCoder::CanMsgPackage((void*)&request, robot::common::ShiftActCtrReq, &canSftActCtrReqMsg);
        write_data(&canSftActCtrReqMsg);
        res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice SftActCtrRequest failed "<< ex.what();
        return false;
    }
    return true;
}

bool Canbus::StrActCtrRequest(canbus::StrActCtrReq::Request &req,
                      canbus::StrActCtrReq::Response &res)
{
    LOG(INFO)<< "controlservice start StrActCtrRequest";
    // StrActCtr request
    TPCANMsg canStrActCtrReqMsg;
    try{
        T_STR_ACT_CTR_REQ request;
        request.iStrAngCtrReq = req.iStrAngCtrReq;
        request.iStrSpdCtrReq = req.iStrSpdCtrReq;
        request.iStrTrqCtrReq = req.iStrTrqCtrReq;
        request.uiStrCtrMdReq = req.uiStrCtrMdReq;

        MessageCoder::CanMsgPackage((void*)&request, robot::common::SteerActCtrReq, &canStrActCtrReqMsg);
        write_data(&canStrActCtrReqMsg);
        res.uiReturn = 1;
    }
    catch (ros::Exception ex)
    {
        res.uiReturn = 0;
        LOG(ERROR)<< "controlservice StrActCtrRequest failed "<< ex.what();
        return false;
    }
    return true;
}








}  // namespace canbus
}  // namespace robot
