/******************************************************************************
 * Copyright 2020 The siasun Transport Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 1.0 (the "License");
 *****************************************************************************/

/**
 * @file
 */

#ifndef ROBOT_ROBOT_LOCALIZATION_H_
#define ROBOT_ROBOT_LOCALIZATION_H_

#include <memory>
#include <string>
#include <utility>
#include <vector>
#include <serial/serial.h>  //ROS已经内置了的串口包
#include "../common/robotapp.h"
#include "ros/ros.h"
#include <stdlib.h>
#include "localizationdata.h"

/**
 * @namespace robot::localization
 * @brief robot::localization
 */
namespace robot {
namespace localizationspace {

/**
* @class localization
*
* @brief localization module main class.
* It processes the control data to send protocol messages to can bus.
*/
class Localization : public robot::common::RobotApp{
 public:
  Localization()  {}
  virtual~Localization() {}

  /**
  * localization总线节点名称
  */
  std::string Name() const override;

  /**
  * @brief module initialization function
  * @return initialization status
  */
  int Init() override;

  /**
  * @brief module start function
  * @return start status
  */
  int Start() override;

  /**
  * @brief module stop function
  */
  void Stop() override;

 private:
  // void PublishChassis();
  // void PublishChassisDetail();
  // void OnControlCommand(const apollo::control::ControlCommand &control_command);
  // apollo::common::Status OnError(const std::string &error_msg);
  // void RegisterCanClients();
  bool IsOpen();

  int GetSerialData();

  int Analyse(const std::string &strSerialResult, NMEA0183 &nmea);

  std::vector<std::string> Split(const std::string& in, const std::string& delim);

  void ParaGPRMC(std::string &nmea_gprmc, GPRMC &gprmc);

  void ParaGPGGA(std::string &nmea_gpgga, GPGGA &gpgga);
  /**
   * @brief string localization ddmm convert to double dd type
   * @param strResult
   * @param pos
   * @return dd type
   */
  inline double StrToDouble(const std::string &strResult, int pos)
  {
      return atof(strResult.substr(0, pos).c_str()) + atof(strResult.substr(pos, strResult.length() - pos).c_str())/60;
  }

  //void Nmea0138Analyse( std::string &serial_data, nmea_msgs::Gpgga &gpgga_msg, nmea_msgs::Gprmc);

  /**
   * @brief publish data message to other nodes
   * @param nmea
   * @return ture or false
   */
  int PublishData(NMEA0183 &nmea);
private:
  int64_t last_timestamp_ = 0;

  ros::Timer timer_;

  std::string serial_port ;

  uint32_t baudrate = 0;

  uint32_t timeout = 0;

  serial::Serial serial_gnss; //声明串口对象

};
}  // namespace localization
}  // namespace robot

#endif  // ROBOT_ROBOT_LOCALIZATION_H_
